from geopytool.ImportDependence import *
from geopytool.CustomClass import *
#from geopytool.TableViewer import TableViewer


class TAS(AppForm):
    _df = pd.DataFrame()
    _changed = False

    xlabel = r'$SiO_2 wt\%$'
    ylabel = r'$Na_2O + K_2O wt\%$'

    itemstocheck = ['SiO2', 'K2O', 'Na2O']
    reference = 'Reference: Maitre, R. W. L., Streckeisen, A., Zanettin, B., Bas, M. J. L., Bonin, B., and Bateman, P., 2004, Igneous Rocks: A Classification and Glossary of Terms: Cambridge University Press, v. -1, no. 70, p. 93–120.'

    ItemNames = ['Foidolite',
                 'Peridotgabbro',
                 'Foid Gabbro',
                 'Foid Monzodiorite',
                 'Foid Monzosyenite',
                 'Foid Syenite',
                 'Gabbro Bs',
                 'Gabbro Ba',
                 'Monzogabbro',
                 'Monzodiorite',
                 'Monzonite',
                 'Syenite',
                 'Quartz Monzonite',
                 'Gabbroic Diorite',
                 'Diorite',
                 'Granodiorite',
                 'Granite',
                 'Quartzolite',
                 ]

    LocationAreas = [[[41, 3], [37, 3], [35, 9], [37, 14], [52.5, 18], [52.5, 14], [48.4, 11.5], [45, 9.4], [41, 7]],
                     [[41, 0], [41, 3], [45, 3], [45, 0]],
                     [[41, 3], [41, 7], [45, 9.4], [49.4, 7.3], [45, 5], [45, 3]],
                     [[45, 9.4], [48.4, 11.5], [53, 9.3], [49.4, 7.3]],
                     [[48.4, 11.5], [52.5, 14], [57.6, 11.7], [53, 9.3]],
                     [[52.5, 14], [52.5, 18], [57, 18], [63, 16.2], [61, 13.5], [57.6, 11.7]],
                     [[45, 0], [45, 2], [52, 5], [52, 0]],
                     [[45, 2], [45, 5], [52, 5]],
                     [[45, 5], [49.4, 7.3], [52, 5]],
                     [[49.4, 7.3], [53, 9.3], [57, 5.9], [52, 5]],
                     [[53, 9.3], [57.6, 11.7], [61, 8.6], [63, 7], [57, 5.9]],
                     [[57.6, 11.7], [61, 13.5], [63, 16.2], [71.8, 13.5], [61, 8.6]],
                     [[61, 8.6], [71.8, 13.5], [69, 8], [63, 7]],
                     [[52, 0], [52, 5], [57, 5.9], [57, 0]],
                     [[57, 0], [57, 5.9], [63, 7], [63, 0]],
                     [[63, 0], [63, 7], [69, 8], [77.3, 0]],
                     [[77.3, 0], [69, 8], [71.8, 13.5], [85.9, 6.8], [87.5, 4.7]],
                     [[77.3, 0], [87.5, 4.7], [90, 4.7], [90, 0]],
                     ]

    AreasHeadClosed = []
    SelectDic = {}

    TypeList=[]

    RittmanList=[]

    All_X = []
    All_Y = []

    whole_labels=[]
    SVM_labels=[]

    def create_main_frame(self):
        self.resize(1000, 800)
        self.main_frame = QWidget()
        self.dpi = 128
        self.fig = Figure((18.0, 12.0), dpi=self.dpi)
        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.2, right=0.7, top=0.9)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        self.axes = self.fig.add_subplot(111)
        self.axes.axis('off')
        # Create the navigation toolbar, tied to the canvas
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls


        self.load_data_button = QPushButton('&Add Data to Compare')
        self.load_data_button.clicked.connect(self.loadDataToTest)


        self.save_button = QPushButton('&Save Img')
        self.save_button.clicked.connect(self.saveImgFile)

        self.result_button = QPushButton('&Classification Result')
        self.result_button.clicked.connect(self.Explain)


        self.save_predict_button_selected = QPushButton('&Predict Result')
        self.save_predict_button_selected.clicked.connect(self.showPredictResultSelected)

        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.TAS)  # int


        self.show_load_data_cb = QCheckBox('&Show Loaded Data')
        self.show_load_data_cb.setChecked(True)
        self.show_load_data_cb.stateChanged.connect(self.TAS)  # int

        self.show_data_index_cb = QCheckBox('&Show Data Index')
        self.show_data_index_cb.setChecked(False)
        self.show_data_index_cb.stateChanged.connect(self.TAS)  # int

        self.irvine_cb = QCheckBox('&Irvine')
        self.irvine_cb.setChecked(True)
        self.irvine_cb.stateChanged.connect(self.TAS)  # int


        self.riedman_cb = QCheckBox('&Rittman')
        self.riedman_cb.setChecked(True)
        self.riedman_cb.stateChanged.connect(self.TAS)  # int


        self.tag_cb = QCheckBox('&Tag')
        self.tag_cb.setChecked(True)
        self.tag_cb.stateChanged.connect(self.TAS)  # int

        self.shape_cb= QCheckBox('&Shape')
        self.shape_cb.setChecked(False)
        self.shape_cb.stateChanged.connect(self.TAS)  # int


        self.hyperplane_cb= QCheckBox('&Hyperplane')
        self.hyperplane_cb.setChecked(False)
        self.hyperplane_cb.stateChanged.connect(self.TAS)  # int


        self.slider_left_label = QLabel('Volcanic')
        self.slider_right_label = QLabel('Plutonic')

        self.slider = QSlider(Qt.Horizontal)
        self.slider.setRange(0, 1)
        self.slider.setValue(0)
        self.slider.setTracking(True)
        self.slider.setTickPosition(QSlider.TicksBothSides)
        self.slider.valueChanged.connect(self.TAS)  # int

        self.kernel_select = QSlider(Qt.Horizontal)
        self.kernel_select.setRange(0, len(self.kernel_list)-1)
        self.kernel_select.setValue(0)
        self.kernel_select.setTracking(True)
        self.kernel_select.setTickPosition(QSlider.TicksBothSides)
        self.kernel_select.valueChanged.connect(self.TAS)  # int
        self.kernel_select_label = QLabel('Kernel')

        #
        # Layout with box sizers
        #
        self.hbox = QHBoxLayout()
        self.hbox1 = QHBoxLayout()
        self.hbox2 = QHBoxLayout()

        for w in [self.load_data_button,self.save_button, self.result_button, self.save_predict_button_selected,self.slider_left_label, self.slider,self.slider_right_label]:
            self.hbox1.addWidget(w)
            self.hbox1.setAlignment(w, Qt.AlignVCenter)

        for w in [self.legend_cb,self.show_load_data_cb,self.show_data_index_cb,self.shape_cb,self.hyperplane_cb,self.kernel_select_label,self.kernel_select ,self.tag_cb,self.irvine_cb ]:
            self.hbox2.addWidget(w)
            self.hbox2.setAlignment(w, Qt.AlignVCenter)



        self.vbox = QVBoxLayout()
        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        self.vbox.addLayout(self.hbox1)
        self.vbox.addLayout(self.hbox2)
        self.textbox = GrowingTextEdit(self)
        self.textbox.setText(self.reference)

        self.vbox.addWidget(self.textbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

        w=self.width()
        h=self.height()


        self.slider_left_label.setFixedWidth(w/10)

        self.slider.setFixedWidth(w/10)

        self.slider_right_label.setFixedWidth(w/10)

    def Irvine(self,x, a = 39.0, b = 3.9492, c = -2.1111, d = 0.86096, e = -0.15188, f = 0.012030, g = -(3.3539 / 10000)):

        return(a+ b*np.power(x,1) +c*np.power(x,2) +d*np.power(x,3) +e*np.power(x,4) +f*np.power(x,5) +g*np.power(x,6))
        pass

    def Rittman(self,x=46,r_1=3.3,r_2=0):
        y_1= r_1*np.sqrt(x-43)
        y_2= r_2*np.sqrt(x-43)
        return(y_1,y_2)


    def loadDataToTest(self):
        TMP =self.getDataFile()
        if TMP != 'Blank':
            self.data_to_test=TMP[0]
        self.TAS()


    def TAS(self, Left=35, Right=79, X0=30, X1=90, X_Gap=7, Base=0,
            Top=19, Y0=1, Y1=19, Y_Gap=19, FontSize=12, xlabel=r'$SiO_2 wt\%$', ylabel=r'$Na_2O + K_2O wt\%$', width=12,
            height=12, dpi=300):



        k_s = int(self.kernel_select.value())
        self.kernel_select_label.setText(self.kernel_list[k_s])


        self.setWindowTitle('TAS (total alkali–silica) diagram Volcanic/Plutonic (Wilson et al. 1989)')
        self.axes.clear()
        #self.axes.axis('off')

        self.FileName_Hint='TAS'
        
        self.axes.set_xlabel(self.xlabel)
        self.axes.set_ylabel(self.ylabel)
        self.axes.spines['right'].set_color('none')
        self.axes.spines['top'].set_color('none')


        self.axes.set_xticks([30,40,50,60,70,80,90])
        self.axes.set_xticklabels([30,40,50,60,70,80,90])

        self.axes.set_yticks([0, 5, 10, 15, 20])
        self.axes.set_yticklabels([0, 5, 10, 15, 20])

        self.axes.set_ylim(bottom=0)



        all_labels=[]
        all_colors=[]
        all_markers=[]
        all_alpha=[]


        self.OutPutData = pd.DataFrame()
        self.LabelList=[]
        self.IndexList=[]
        self.TypeList=[]
        self.RittmanList=[]



        for i in range(len(self._df)):
            target = self._df.at[i, 'Label']
            color = self._df.at[i, 'Color']
            marker = self._df.at[i, 'Marker']
            alpha = self._df.at[i, 'Alpha']

            if target not in all_labels:
                all_labels.append(target)
                all_colors.append(color)
                all_markers.append(marker)
                all_alpha.append(alpha)

        self.whole_labels = all_labels


        YIrvine= np.arange(0,10.2,0.1)
        XIrvine= self.Irvine(YIrvine)


        PointLabels = []
        PointColors = []
        x = []
        y = []
        Locations = [(39, 10), (43, 1.5), (44, 6), (47.5, 3.5), (49.5, 1.5), (49, 5.2), (49, 9.5), (54, 3), (53, 7),
                     (53, 12),
                     (60, 4),
                     (57, 8.5), (57, 14), (67, 5), (65, 12), (67, 9), (75, 9), (85, 1), (55, 18.5)]
        X_offset = -6
        Y_offset = 3

        if (int(self.slider.value())==0):
            Labels = [u'F', u'Pc', u'U1', u'Ba', u'Bs', u'S1', u'U2', u'O1', u'S2', u'U3', u'O2', u'S3', u'Ph', u'O3',
                      u'T',
                      u'Td', u'R', u'Q', u'S/N/L']
            title = 'TAS (total alkali–silica) diagram Volcanic (after Wilson et al. 1989).'
            description = '\n' \
                          'F: Foidite, Ph: Phonolite, Pc Pocrobasalt, U1: Tephrite (ol < 10%) Basanite(ol > 10%), U2: Phonotephrite, U3: Tephriphonolite,\n' \
                          'Ba: alkalic basalt,Bs: subalkalic baslt, S1: Trachybasalt, S2: Basaltic Trachyandesite, S3: Trachyandesite,\n' \
                          'O1: Basaltic Andesite, O2: Andesite, O3 Dacite, T: Trachyte , Td: Trachydacite , R: Rhyolite, Q: Silexite \n' \
                          'S/N/L: Sodalitite/Nephelinolith/Leucitolith'
            self.setWindowTitle(title)
            self.textbox.setText(self.reference+description)




        else:
            Labels = [u'F', u'Pc', u'U1', u'Ba', u'Bs', u'S1', u'U2', u'O1', u'S2', u'U3', u'O2', u'S3', u'Ph', u'O3',
                      u'T',
                      u'Td', u'R', u'Q', u'T/U/I']
            title = 'TAS (total alkali–silica) diagram Plutonic (after Wilson et al. 1989).'
            description = '\n' \
                          'F: Foidolite, Ph: Foid Syenite, Pc: Peridotgabbro, U1: Foid Gabbro, U2: Foid Monzodiorite, U3: Foid Monzosyenite,\n' \
                          'Ba: alkalic gabbro,Bs: subalkalic gabbro, S1: Monzogabbro, S2: Monzodiorite, S3: Monzonite,\n' \
                          'O1: Gabbroic Diorite, O2: Diorite, O3: Graodiorite, T: Syenite , Td: Quartz Monzonite , R: Granite, Q: Quartzolite \n' \
                          'T/U/I: Tawite/Urtite/Italite'


            self.setWindowTitle(title)
            self.textbox.setText(self.reference+description)


        TagNumber = min(len(Labels), len(Locations))
        if (self.tag_cb.isChecked()):
            for k in range(TagNumber):
                self.axes.annotate(Labels[k], Locations[k], xycoords='data', xytext=(X_offset, Y_offset),
                                   textcoords='offset points',
                                   fontsize=9, color='grey', alpha=0.8)

        self.DrawLine([(41, 0), (41, 3), (45, 3)])
        self.DrawLine([(45, 0), (45, 3), (45, 5), (49.4, 7.3), (53, 9.3), (57.6, 11.7), (61, 13.5), (63, 16.2)], )
        self.DrawLine([(52, 5), (57, 5.9), (63, 7), (69, 8), (71.8, 13.5), (61, 8.6)])
        self.DrawLine([(45, 2), (45, 5), (52, 5), (45, 2)])
        self.DrawLine(
            [(69, 8), (77.3, 0), (87.5, 4.7), (85.9, 6.8), (71.8, 13.5), (63, 16.2), (57, 18), (52.5, 18), (37, 14),
             (35, 9), (37, 3), (41, 3)])

        self.DrawLine([(63, 0), (63, 7), (57.6, 11.7), (52.5, 14), (52.5, 18)])
        self.DrawLine([(57, 0), (57, 5.9), (53, 9.3), (48.4, 11.5)])
        self.DrawLine([(52, 0), (52, 5), (49.4, 7.3), (45, 9.4)])
        self.DrawLine([(41, 3), (41, 7), (45, 9.4)])

        self.DrawLine([(45, 9.4), (48.4, 11.5), (52.5, 14)])

        # self.DrawLine([(41.75, 1), (52.5, 5)])
        # self.DrawLine([(45.85, 2.75), (46.85, 3.0), (50.0, 4.0), (53.1, 5.0), (55.0, 5.8), (55.6, 6.0), (60.0, 6.8),(61.5, 7.0), (65.0, 7.35), (70.0, 7.85), (71.6, 8.0), (75.0, 8.3), (76.4, 8.4)])
        # self.DrawLine([(45.85, 2.75), (46.85, 3.0), (50.0, 4.0), (53.1, 5.0), (55.0, 5.8), (55.6, 6.0), (60.0, 6.8),(61.5, 7.0), (65.0, 7.35), (70.0, 7.85), (71.6, 8.0), (75.0, 8.3), (76.4, 8.4)])
        # self.DrawLine([(39.8, 0.35), (65.6, 9.7)])
        # self.DrawLine([(39.2, 0.0), (40.0, 0.4), (43.2, 2.0), (45.0, 2.8), (48.0, 4.0), (50.0, 4.75), (53.7, 6.0),(55.0, 6.4), (60.0, 8.0), (65.0, 8.8)])

        self.Check()

        if self.OutPutCheck==True:
            pass

        if (self._changed):
            df =  self.CleanDataFile(self._df)

            for i in range(len(df)):
                TmpLabel = ''
                if (df.at[i, 'Label'] in PointLabels or df.at[i, 'Label'] == ''):
                    TmpLabel = ''
                else:
                    PointLabels.append(df.at[i, 'Label'])
                    TmpLabel = df.at[i, 'Label']


                TmpColor = ''
                if (df.at[i, 'Color'] in PointColors or df.at[i, 'Color'] == ''):
                    TmpColor = ''
                else:
                    PointColors.append(df.at[i, 'Color'])
                    TmpColor = df.at[i, 'Color']



                x.append(df.at[i, 'SiO2'])
                y.append(df.at[i, 'Na2O'] + df.at[i, 'K2O'])
                Size = df.at[i, 'Size']
                Color = df.at[i, 'Color']

                # print(Color, df.at[i, 'SiO2'], (df.at[i, 'Na2O'] + df.at[i, 'K2O']))

                Alpha = df.at[i, 'Alpha']
                Marker = df.at[i, 'Marker']
                Label = df.at[i, 'Label']

                xtest=df.at[i, 'SiO2']
                ytest=df.at[i, 'Na2O'] + df.at[i, 'K2O']



                for j in self.ItemNames:
                    if self.SelectDic[j].contains_point([xtest,ytest]):

                        self.LabelList.append(Label)
                        self.TypeList.append(j)

                        self.RittmanList.append(round( ytest**2/(xtest-43) + 0.001, 2))

                        if 'Index' in self._df_back.columns.values:
                            self.IndexList.append(self._df_back.at[i, 'Index'])
                        else:
                            self.IndexList.append('No ' + str(i))
                        break
                    pass


                self.axes.scatter(df.at[i, 'SiO2'], (df.at[i, 'Na2O'] + df.at[i, 'K2O']), marker=df.at[i, 'Marker'],
                  s=df.at[i, 'Size'], color=df.at[i, 'Color'], alpha=df.at[i, 'Alpha'], label=TmpLabel)
                  #edgecolors='black')


            XtoFit = {}
            YtoFit = {}

            SVM_X=[]
            SVM_Y=[]

            for i in  PointLabels:
                XtoFit[i]=[]
                YtoFit[i]=[]


            for i in range(len(df)):
                Alpha = df.at[i, 'Alpha']
                Marker = df.at[i, 'Marker']
                Label = df.at[i, 'Label']

                xtest=df.at[i, 'SiO2']
                ytest=df.at[i, 'Na2O'] + df.at[i, 'K2O']

                XtoFit[Label].append(xtest)
                YtoFit[Label].append(ytest)

                SVM_X.append(xtest)
                SVM_Y.append(ytest)

            if (self.shape_cb.isChecked()):
                for i in PointLabels:

                    if XtoFit[i] != YtoFit[i]:
                        xmin, xmax = min(XtoFit[i]), max(XtoFit[i])
                        ymin, ymax = min(YtoFit[i]), max(YtoFit[i])

                        DensityColorMap = 'Greys'
                        DensityAlpha = 0.1

                        DensityLineColor = PointColors[PointLabels.index(i)]
                        DensityLineAlpha = 0.3

                        # Peform the kernel density estimate
                        xx, yy = np.mgrid[xmin:xmax:200j, ymin:ymax:200j]
                        # print(self.ShapeGroups)
                        # command='''xx, yy = np.mgrid[xmin:xmax:'''+str(self.ShapeGroups)+ '''j, ymin:ymax:''' +str(self.ShapeGroups)+'''j]'''
                        # exec(command)
                        # print(xx, yy)
                        positions = np.vstack([xx.ravel(), yy.ravel()])
                        values = np.vstack([XtoFit[i], YtoFit[i]])
                        kernelstatus = True
                        try:
                            st.gaussian_kde(values)
                        except Exception as e:
                            self.ErrorEvent(text=repr(e))
                            kernelstatus = False
                        if kernelstatus == True:
                            kernel = st.gaussian_kde(values)
                            f = np.reshape(kernel(positions).T, xx.shape)
                            # Contourf plot
                            cfset = self.axes.contourf(xx, yy, f, cmap=DensityColorMap, alpha=DensityAlpha)
                            ## Or kernel density estimate plot instead of the contourf plot
                            # self.axes.imshow(np.rot90(f), cmap='Blues', extent=[xmin, xmax, ymin, ymax])
                            # Contour plot
                            cset = self.axes.contour(xx, yy, f, colors=DensityLineColor, alpha=DensityLineAlpha)
                            # Label plot
                            #self.axes.clabel(cset, inline=1, fontsize=10)


            if (self.irvine_cb.isChecked()):
                self.axes.plot(XIrvine, YIrvine,color= 'black', linewidth=1,
                           linestyle=':', alpha=0.6,label='Irvine, Barragar 1971\n')



            if (len(self.data_to_test) > 0):
                self.data_to_test_back= self.data_to_test
                contained = True
                missing = 'Miss setting infor:'

                for i in ['Label', 'Color', 'Marker', 'Alpha']:
                    if i not in self.data_to_test.columns.values.tolist():
                        contained = False
                        missing = missing + '\n' + i

                if contained == True:
                    for i in self.data_to_test.columns.values.tolist():
                        if i not in self._df.columns.values.tolist():
                            self.data_to_test = self.data_to_test.drop(columns=i)

                    # print(self.data_to_test)

                    test_labels = []
                    test_colors = []
                    test_markers = []
                    test_alpha = []

                    for i in range(len(self.data_to_test)):

                        # print(self.data_to_test.at[i, 'Label'])
                        target = self.data_to_test.at[i, 'Label']
                        color = self.data_to_test.at[i, 'Color']
                        marker = self.data_to_test.at[i, 'Marker']
                        alpha = self.data_to_test.at[i, 'Alpha']

                        if target not in test_labels and target not in all_labels:
                            test_labels.append(target)
                            test_colors.append(color)
                            test_markers.append(marker)
                            test_alpha.append(alpha)

                    self.whole_labels = self.whole_labels + test_labels


                    self.load_settings_backup = self.data_to_test
                    Load_ItemsToTest = ['Label', 'Number', 'Tag', 'Name', 'Author', 'DataType', 'Marker', 'Color',
                                        'Size',
                                        'Alpha',
                                        'Style', 'Width']
                    for i in self.data_to_test.columns.values.tolist():
                        if i not in Load_ItemsToTest:
                            self.load_settings_backup = self.load_settings_backup.drop(i, 1)

                    print(self.load_settings_backup, self.data_to_test)

                    print(self.load_settings_backup.shape, self.data_to_test.shape)


                    try:

                        for i in range(len(self.data_to_test)):

                            target = self.data_to_test.at[i, 'Label']
                            if target not in all_labels:
                                all_labels.append(target)
                                tmp_label = self.data_to_test.at[i, 'Label']
                            else:
                                tmp_label=''

                            x_load_test = self.data_to_test.at[i, 'SiO2']
                            y_load_test = self.data_to_test.at[i, 'Na2O'] + self.data_to_test.at[i, 'K2O']



                            for j in self.ItemNames:
                                if self.SelectDic[j].contains_point([x_load_test, y_load_test]):
                                    self.LabelList.append(self.data_to_test.at[i, 'Label'])
                                    self.TypeList.append(j)


                                    print(self.data_to_test.at[i, 'Label'],j)
                                    self.RittmanList.append(round(y_load_test ** 2 / (x_load_test - 43) + 0.001, 2))

                                    if 'Index' in self.data_to_test_back.columns.values:
                                        self.IndexList.append(self.data_to_test_back.at[i, 'Index'])
                                    else:
                                        self.IndexList.append('No '+str(i))
                                    break

                            if (self.show_load_data_cb.isChecked()):

                                self.axes.scatter(self.data_to_test.at[i, 'SiO2'], (
                                        self.data_to_test.at[i, 'Na2O'] + self.data_to_test.at[i, 'K2O']),
                                                  marker=self.data_to_test.at[i, 'Marker'],
                                                  s=self.data_to_test.at[i, 'Size'],
                                                  color=self.data_to_test.at[i, 'Color'],
                                                  alpha=self.data_to_test.at[i, 'Alpha'],
                                                  label=tmp_label)
                                                  #edgecolors='black')


                    except Exception as e:
                        self.ErrorEvent(text=repr(e))


            if (self.legend_cb.isChecked()):
                self.axes.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0, prop=fontprop)


            self.All_X=SVM_X
            self.All_Y=SVM_Y

            if (self.hyperplane_cb.isChecked()):
                clf = svm.SVC(C=1.0, kernel=self.kernel_list[k_s], probability=True)

                svm_x= SVM_X
                svm_y= SVM_Y

                print(len(svm_x),len(svm_y),len(df.index))

                xx, yy = np.meshgrid(np.arange(min(svm_x), max(svm_x), np.ptp(svm_x) / 500),
                                     np.arange(min(svm_y), max(svm_y), np.ptp(svm_y) / 500))

                le = LabelEncoder()
                le.fit(self._df.Label)
                class_label = le.transform(self._df.Label)
                svm_train= pd.concat([pd.DataFrame(svm_x),pd.DataFrame(svm_y)], axis=1)
                svm_train=svm_train.values
                clf.fit(svm_train,class_label)
                Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
                Z = Z.reshape(xx.shape)
                self.axes.contourf(xx, yy, Z, cmap='hot', alpha=0.2)

            if (self.show_data_index_cb.isChecked()):

                if 'Index' in self._df.columns.values:

                    for i in range(len(self._df)):
                        self.axes.annotate(self._df.at[i, 'Index'],
                                           xy=(self.All_X[i],
                                               self.All_Y[i]),
                                           color=self._df.at[i, 'Color'],
                                           alpha=self._df.at[i, 'Alpha'])
                else:
                    for i in range(len(self._df)):
                        self.axes.annotate('No' + str(i + 1),
                                           xy=(self.All_X[i],
                                               self.All_Y[i]),
                                           color=self._df.at[i, 'Color'],
                                           alpha=self._df.at[i, 'Alpha'])


            self.canvas.draw()





        self.OutPutTitle='TAS'

        self.OutPutData = pd.DataFrame(
            {'Label': self.LabelList,
             'Index':self.IndexList,
             'RockType': self.TypeList,
             'Rittman Index': self.RittmanList
             })

        self.OutPutFig=self.fig


    def showPredictResultSelected(self):

        k_s = int(self.kernel_select.value())
        try:
            clf = svm.SVC(C=1.0, kernel=self.kernel_list[k_s], probability=True)
            svm_x = self.All_X
            svm_y = self.All_Y
            le = LabelEncoder()
            le.fit(self._df.Label)
            class_label = le.transform(self._df.Label)
            svm_train = pd.concat([pd.DataFrame(svm_x), pd.DataFrame(svm_y)], axis=1)
            svm_train = svm_train.values
            clf.fit(svm_train, self._df.Label)
            xx = self.data_to_test['SiO2']
            yy=[]
            for k in range(len(self.data_to_test)):
                tmp =self.data_to_test.at[k, 'Na2O'] + self.data_to_test.at[k,'K2O']
                yy.append(tmp)
                pass

            Z = clf.predict(np.c_[xx.ravel(), yy])
            Z2 = clf.predict_proba(np.c_[xx.ravel(), yy])

            for i in Z2:
                print(i)

            proba_df = pd.DataFrame(Z2)
            proba_df.columns = clf.classes_


            proba_list = []
            for i in range(len(proba_df)):
                proba_list.append(round(max(proba_df.iloc[i])+ 0.001, 2))
            predict_result = pd.concat(
                [self.data_to_test['Label'], pd.DataFrame({'SVM Classification': Z}), pd.DataFrame({'Confidence probability': proba_list})],
                axis=1).set_index('Label')
            print(predict_result)


            self.predictpop = TableViewer(df=predict_result, title='SVM Predict Result')
            self.predictpop.show()

            '''
            DataFileOutput, ok2 = QFileDialog.getSaveFileName(self, '文件保存', 'C:/',  'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出
            if (DataFileOutput != ''):
                if ('csv' in DataFileOutput):
                    # DataFileOutput = DataFileOutput[0:-4]
                    predict_result.to_csv(DataFileOutput, sep=',', encoding='utf-8')
                    # self.result.to_csv(DataFileOutput + '.csv', sep=',', encoding='utf-8')
                elif ('xlsx' in DataFileOutput):
                    # DataFileOutput = DataFileOutput[0:-5]
                    predict_result.to_excel(DataFileOutput, encoding='utf-8')
                    # self.result.to_excel(DataFileOutput + '.xlsx', encoding='utf-8')


            '''

        except Exception as e:
            msg = 'You need to load another data to run SVM.\n '
            self.ErrorEvent(text= msg +repr(e) )




    def Explain(self):

        #self.OutPutData = self.OutPutData.set_index('Label')

        self.tablepop = TableViewer(df=self.OutPutData,title='TAS Result')
        self.tablepop.show()


from geopytool.ImportDependence import *
from geopytool.CustomClass import *

class Pearce(AppForm):
    reference = 'Reference: Pearce, J. A., Harris, N. B. W., and Tindle, A. G., 1984, Trace Element Discrimination Diagrams for the Tectonic Interpretation of Granitic Rocks: Journal of Petrology, v. 25, no. 4, p. 956-983.'
    Lines = []
    Tags = []
    title = 'Pearce diagram (after Julian A. Pearce et al., 1984).'

    description = '\n syn-COLG: syn-collision granites\t VAG: volcanic arc granites\n WPG: within plate granites\t ORG: ocean ridge granites  \n '

    itemstocheck = [u'Y', u'Yb', u'Nb', u'Rb', u'Ta']



    text = [u'0.1', u'1', u'10', u'100', u'1000', u'10000', u'100000', u'1000000', u'10000000']

    Condation0 = {'BaseLines': [[(2, 80), (55, 300)],
                                [(55, 300), (400, 2000)],
                                [(55, 300), (51.5, 8)],
                                [(51.5, 8), (50, 1)],
                                [(51.5, 8), (2000, 400)], ],
                  'xLabel': r'Y+Nb (PPM)',
                  'yLabel': r'Rb (PPM)',
                  'Labels': [u'syn-COLG', u'VAG', u'WPG', u'ORG'],
                  'Locations': [(1, 3), (1, 1), (2.4, 2.4), (3, 1)],
                  }

    Condation1 = {'BaseLines': [[(0.5, 140), (6, 200)],
                                [(6, 200), (50, 2000)],
                                [(6, 200), (6, 8)],
                                [(6, 8), (6, 1)],
                                [(6, 8), (200, 400)], ],
                  'xLabel': r'Yb+Ta (PPM)',
                  'yLabel': r'Rb (PPM)',
                  'Labels': [u'syn-COLG', u'VAG', u'WPG', u'ORG'],
                  'Locations': [(0.5, 3), (0.5, 1), (1.5, 2.4), (2, 1)],
                  }

    Condation2 = {'BaseLines': [[(1, 2000), (50, 10)],
                                [(40, 1), (50, 10)],
                                [(50, 10), (1000, 100)],
                                [(25, 25), (1000, 400)], ],
                  'xLabel': r'Y (PPM)',
                  'yLabel': r'Nb (PPM)',
                  'Labels': [u'syn-COLG', u'VAG', u'WPG', u'ORG'],
                  'Locations': [(0.5, 1.5), (0.5, 2), (2, 2), (2.2, 0.5)],
                  }

    Condation3 = {'BaseLines': [[(0.55, 20), (3, 2)],
                                [(0.1, 0.35), (3, 2)],
                                [(3, 2), (5, 1)],
                                [(5, 0.05), (5, 1)],
                                [(5, 1), (100, 7)],
                                [(3, 2), (100, 20)], ],
                  'xLabel': r'Yb (PPM)',
                  'yLabel': r'Ta (PPM)',
                  'Labels': [u'syn-COLG', u'VAG', u'WPG', u'ORG'],
                  'Locations': [(-0.5, 0.1), (-0.5, -1), (0.7, 1), (1.5, 0)],
                  }

    condation = [Condation0, Condation1, Condation2, Condation3]

    def __init__(self, parent=None, df=pd.DataFrame()):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle(self.title)

        self._df = df
        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to Pearce')

        self.create_main_frame()
        self.create_status_bar()

    def create_main_frame(self):
        self.resize(900, 900)
        self.main_frame = QWidget()
        self.dpi = 128
        self.fig ,self.axes= plt.subplots(2, 2,figsize=(16.0, 16.0),dpi=self.dpi)
        self.fig.subplots_adjust(hspace=0.5, wspace=0.5,left=0.1, bottom=0.2, right=0.8, top=0.9)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        # Create the navigation toolbar, tied to the canvas
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls
        self.save_button = QPushButton('&Save')
        self.save_button.clicked.connect(self.saveImgFile)


        #self.result_button = QPushButton('&Result')
        #self.result_button.clicked.connect(self.Pearce)


        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.Pearce)  # int


        self.detail_cb = QCheckBox('&Detail')
        self.detail_cb.setChecked(True)
        self.detail_cb.stateChanged.connect(self.Pearce)  # int



        #
        # Layout with box sizers
        #
        self.hbox = QHBoxLayout()

        for w in [self.save_button, self.legend_cb,  self.detail_cb]:
            self.hbox.addWidget(w)
            self.hbox.setAlignment(w, Qt.AlignVCenter)

        self.vbox = QVBoxLayout()
        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        self.vbox.addLayout(self.hbox)


        self.textbox = GrowingTextEdit(self)
        self.textbox.setText(self.reference+self.description)

        self.vbox.addWidget(self.textbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)


    def customDrawLine(self, l=[(41, 0), (41, 3), (45, 3)], color='grey', linewidth=0.5, linestyle='-', linelabel='',
                 alpha=0.5):
        x = []
        y = []

        label = linelabel

        for i in l:
            x.append(i[0])
            y.append(i[1])

        return (x, y)

    def customDrawLogLine(self, l=[(41, 0), (41, 3), (45, 3)], color='grey', linewidth=0.5, linestyle='-', linelabel='',
                    alpha=0.5):
        x = []
        y = []
        for i in l:
            x.append(math.log(i[0], 10))
            y.append(math.log(i[1], 10))

        return (x, y)


    def Pearce(self):

        self.WholeData = []

        #raw = self._df

        raw = self.CleanDataFile(self._df)


        self.axes[0, 0].clear()
        #self.axes[0, 0].axis('off')

        self.axes[0, 0].set_xlabel(self.condation[0]['xLabel'])
        self.axes[0, 0].set_ylabel(self.condation[0]['yLabel'])

        self.axes[0, 0].set_xlim(0.3, 3.2)
        self.axes[0, 0].set_xticks([ 1, 2, 3])
        self.axes[0, 0].set_xticklabels([ 10,  100, 1000])

        self.axes[0, 0].set_ylim(0, 3.2)
        self.axes[0, 0].set_yticks([0, 1, 2, 3])
        self.axes[0, 0].set_yticklabels([1, 10,  100, 1000])

        self.axes[0, 0].spines['right'].set_color('none')
        self.axes[0, 0].spines['top'].set_color('none')


        self.axes[0, 1].clear()
        #self.axes[0, 1].axis('off')

        self.axes[0, 1].set_xlabel(self.condation[1]['xLabel'])
        self.axes[0, 1].set_ylabel(self.condation[1]['yLabel'])

        self.axes[0, 1].set_xlim(-0.2, 2.5)
        self.axes[0, 1].set_xticks([0, 1, 2])
        self.axes[0, 1].set_xticklabels([1, 10, 100])

        self.axes[0, 1].set_ylim(0, 3.2)
        self.axes[0, 1].set_yticks([0, 1, 2, 3])
        self.axes[0, 1].set_yticklabels([1, 10, 100, 1000])

        self.axes[0, 1].spines['right'].set_color('none')
        self.axes[0, 1].spines['top'].set_color('none')




        self.axes[1,0].clear()
        #self.axes[1,0].axis('off')

        self.axes[1,0].set_xlabel(self.condation[2]['xLabel'])
        self.axes[1,0].set_ylabel(self.condation[2]['yLabel'])



        self.axes[1, 0].set_xlim(0, 3.2)
        self.axes[1, 0].set_xticks([0, 1, 2, 3])
        self.axes[1, 0].set_xticklabels([1, 10,  100, 1000])

        self.axes[1, 0].set_ylim(0, 3.2)
        self.axes[1, 0].set_yticks([0, 1, 2, 3])
        self.axes[1, 0].set_yticklabels([1, 10,  100, 1000])

        self.axes[1, 0].spines['right'].set_color('none')
        self.axes[1, 0].spines['top'].set_color('none')


        self.axes[1,1].clear()
        #self.axes[1,1].axis('off')

        self.axes[1,1].set_xlabel(self.condation[3]['xLabel'])
        self.axes[1,1].set_ylabel(self.condation[3]['yLabel'])



        self.axes[1, 1].set_xlim(-1, 2)
        self.axes[1, 1].set_xticks([-1, 0, 1, 2])
        self.axes[1, 1].set_xticklabels([0.1, 1, 10,  100])

        self.axes[1, 1].set_ylim(-1.2, 2)
        self.axes[1, 1].set_yticks([-1, 0, 1, 2])
        self.axes[1, 1].set_yticklabels([0.1, 1, 10,  100])

        self.axes[1, 1].spines['right'].set_color('none')
        self.axes[1, 1].spines['top'].set_color('none')








        BaseLinesA = self.condation[0]['BaseLines']

        for i in BaseLinesA:
            #self.DrawLogLine(l=i)
            self.axes[0, 0].plot(self.customDrawLogLine(l=i)[0],self.customDrawLogLine(l=i)[1], color='black', linewidth=0.8, alpha=0.5)



        self.TagsA = []

        for i in range(len(self.condation[0]['Labels'])):
            self.TagsA.append(Tag(Label=self.condation[0]['Labels'][i], Location=self.condation[0]['Locations'][i]))


        BaseLinesB = self.condation[1]['BaseLines']

        for i in BaseLinesB:
            #self.DrawLogLine(l=i)
            self.axes[0, 1].plot(self.customDrawLogLine(l=i)[0],self.customDrawLogLine(l=i)[1], color='black', linewidth=0.8, alpha=0.5)



        self.TagsB = []

        for i in range(len(self.condation[1]['Labels'])):
            self.TagsB.append(Tag(Label=self.condation[1]['Labels'][i], Location=self.condation[1]['Locations'][i]))


        BaseLinesC = self.condation[2]['BaseLines']

        for i in BaseLinesC:
            #self.DrawLogLine(l=i)

            if (i==   [(25, 25), (1000, 400)]):
                self.axes[1, 0].plot(self.customDrawLogLine(l=i)[0], self.customDrawLogLine(l=i)[1],linestyle=':', color='grey',
                                     linewidth=0.8, alpha=0.3)

            else:
                self.axes[1, 0].plot(self.customDrawLogLine(l=i)[0],self.customDrawLogLine(l=i)[1], color='black', linewidth=0.8, alpha=0.5)



        self.TagsC = []

        for i in range(len(self.condation[2]['Labels'])):
            self.TagsC.append(Tag(Label=self.condation[2]['Labels'][i], Location=self.condation[2]['Locations'][i]))


        BaseLinesD = self.condation[3]['BaseLines']

        for i in BaseLinesD:
            #self.DrawLogLine(l=i)
            if (i==  [(3, 2), (100, 20)]):
                self.axes[1, 1].plot(self.customDrawLogLine(l=i)[0], self.customDrawLogLine(l=i)[1],linestyle=':', color='grey',
                                     linewidth=0.8, alpha=0.3)

            else:
                self.axes[1, 1].plot(self.customDrawLogLine(l=i)[0],self.customDrawLogLine(l=i)[1], color='black', linewidth=0.8, alpha=0.5)





        PointLabels = []

        self.TagsD = []

        for i in range(len(self.condation[3]['Labels'])):
            self.TagsD.append(Tag(Label=self.condation[3]['Labels'][i], Location=self.condation[3]['Locations'][i]))


        for i in range(len(raw)):
            # raw.at[i, 'DataType'] == 'User' or raw.at[i, 'DataType'] == 'user' or raw.at[i, 'DataType'] == 'USER'

            TmpLabel = ''

            #   self.WholeData.append(math.log(tmp, 10))

            if (raw.at[i, 'Label'] in PointLabels or raw.at[i, 'Label'] == ''):
                TmpLabel = ''
            else:
                PointLabels.append(raw.at[i, 'Label'])
                TmpLabel = raw.at[i, 'Label']

            xa, ya = 0, 0
            xb, yb = 0, 0
            xc, yc = 0, 0
            xd, yd = 0, 0

            xa, ya = (raw.at[i, 'Y'] + raw.at[i, 'Nb']), raw.at[i, 'Rb']
            xb, yb = (raw.at[i, 'Yb'] + raw.at[i, 'Ta']), raw.at[i, 'Rb']
            xc, yc = raw.at[i, 'Y'], raw.at[i, 'Nb']
            xd, yd = raw.at[i, 'Yb'], raw.at[i, 'Ta']

            try:
                self.axes[0, 0].scatter(math.log(xa, 10), math.log(ya, 10), marker=raw.at[i, 'Marker'],
                                    s=raw.at[i, 'Size'], color=raw.at[i, 'Color'], alpha=raw.at[i, 'Alpha'],
                                    label=TmpLabel, edgecolors='black')
            except:
                pass

            try:
                self.axes[0, 1].scatter(math.log(xb, 10), math.log(yb, 10), marker=raw.at[i, 'Marker'],
                                    s=raw.at[i, 'Size'], color=raw.at[i, 'Color'], alpha=raw.at[i, 'Alpha'],
                                    label=TmpLabel, edgecolors='black')
            except:
                pass

            try:
                self.axes[1, 0].scatter(math.log(xc, 10), math.log(yc, 10), marker=raw.at[i, 'Marker'],
                                    s=raw.at[i, 'Size'], color=raw.at[i, 'Color'], alpha=raw.at[i, 'Alpha'],
                                    label=TmpLabel, edgecolors='black')
            except:
                pass

            try:
                self.axes[1, 1].scatter(math.log(xd, 10), math.log(yd, 10), marker=raw.at[i, 'Marker'],
                                    s=raw.at[i, 'Size'], color=raw.at[i, 'Color'], alpha=raw.at[i, 'Alpha'],
                                    label=TmpLabel, edgecolors='black')
            except:
                pass




        Tale = 0
        Head = 0

        if (len(self.WholeData) > 0):
            Tale = min(self.WholeData)
            Head = max(self.WholeData)

        Location = round(Tale - (Head - Tale) / 5)

        count = round((Head - Tale) / 5 * 7)


        if (self.legend_cb.isChecked()):
            self.axes[0, 1].legend(bbox_to_anchor=(1.05, 1),loc=2, borderaxespad=0, prop=fontprop)



        if (self.detail_cb.isChecked()):
            for i in self.TagsA:
                self.axes[0,0].annotate(i.Label, xy=i.Location, xycoords='data', xytext=(i.X_offset, i.Y_offset),
                                   textcoords='offset points',
                                   fontsize=8, color='grey', alpha=0.8)

            for i in self.TagsB:
                self.axes[0,1].annotate(i.Label, xy=i.Location, xycoords='data', xytext=(i.X_offset, i.Y_offset),
                                   textcoords='offset points',
                                   fontsize=8, color='grey', alpha=0.8)

            for i in self.TagsC:
                self.axes[1,0].annotate(i.Label, xy=i.Location, xycoords='data', xytext=(i.X_offset, i.Y_offset),
                                   textcoords='offset points',
                                   fontsize=8, color='grey', alpha=0.8)

            for i in self.TagsD:
                self.axes[1,1].annotate(i.Label, xy=i.Location, xycoords='data', xytext=(i.X_offset, i.Y_offset),
                                   textcoords='offset points',
                                   fontsize=8, color='grey', alpha=0.8)




        self.canvas.draw()


        self.OutPutTitle='Pearce'

        self.OutPutFig=self.fig



#coding:utf-8
import matplotlib
matplotlib.use('Qt5Agg')

import importlib

'''
try:
    importlib.import_module(package)
except ImportError:
    import pip

    pip.main(['install', package])
finally:
    globals()[package] = importlib.import_module(package)
'''

import pip
import subprocess

import webbrowser
import sys
import sklearn as sk
#import vispy.app, vispy.visuals, vispy.scene,vispy.plot, vispy.io, vispy.color
#import sklearn_pandas as skpd

import scipy.stats as st



from scipy.optimize import leastsq
from scipy.optimize import curve_fit
import requests
import re
import random
import pandas as pd
import os
import numpy as np
import matplotlib.pyplot as plt

from matplotlib import path
import matplotlib.patches as patches
import matplotlib.image as mpimg
import matplotlib.font_manager as font_manager
import matplotlib.backends.backend_pdf
import math
import csv
from xml.dom import minidom
from sklearn.svm import SVC
from sklearn.preprocessing import StandardScaler, MinMaxScaler, Normalizer, Binarizer,LabelEncoder, OneHotEncoder, Imputer,  PolynomialFeatures, FunctionTransformer
from sklearn.neighbors import NearestNeighbors,KernelDensity
from sklearn.feature_selection import VarianceThreshold, SelectKBest, chi2
from sklearn.metrics.pairwise import euclidean_distances
from sklearn.decomposition import PCA, FastICA,FactorAnalysis
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis

from sklearn import svm

from sklearn import datasets
from scipy.stats import mode,gaussian_kde
from scipy.spatial import distance
from scipy.cluster.hierarchy import dendrogram, linkage
from scipy.cluster import hierarchy as hc
from scipy.signal import find_peaks,find_peaks_cwt,fftconvolve

from itertools import product, combinations, permutations, repeat

from PyQt5.QtWidgets import QMainWindow, QMenu, QSizePolicy, QMessageBox, QWidget, QFileDialog, QAction,QTextEdit, QLineEdit,    QApplication, QPushButton, QSlider, QLabel, QHBoxLayout, QVBoxLayout,QProxyStyle,QStyle,qApp,QCheckBox
from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtCore import Qt
from PyQt5 import QtCore, QtGui, QtWidgets
from numpy import vstack, array, nan, mean, median, ptp, var, std, cov, corrcoef, arange, sin, pi ,nanmean, nanmedian, nanvar, nanstd
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.font_manager import ttfFontProperty
from matplotlib.figure import Figure
from matplotlib import colors
from matplotlib import cm
from matplotlib.patches import Rectangle
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5 import NavigationToolbar2QT as NavigationToolbar
from matplotlib import ft2font
from bs4 import BeautifulSoup
#from geopytool.TableViewer import TableViewer


LocationOfMySelf=os.path.dirname(__file__)

#print(LocationOfMySelf,'Import Denpendence')

fpath = LocationOfMySelf+('/wqy.ttf')

font = ft2font.FT2Font(fpath)
fprop = font_manager.FontProperties(fname=fpath)

ttfFontProp = ttfFontProperty(font)
fontprop = font_manager.FontProperties(family='sans-serif',
                            size=9,
                            fname=ttfFontProp.fname,
                            stretch=ttfFontProp.stretch,
                            style=ttfFontProp.style,
                            variant=ttfFontProp.variant,
                            weight=ttfFontProp.weight)


'''
fontprop = font_manager.FontProperties(family='sans-serif',
                            size=9,
                            fname=ttfFontProp.fname,
                            stretch=ttfFontProp.stretch,
                            style=ttfFontProp.style,
                            variant=ttfFontProp.variant,
                            weight=ttfFontProp.weight)
'''

plt.rcParams['svg.fonttype'] = 'none'
plt.rcParams['pdf.fonttype'] = 'truetype'
plt.rcParams['axes.unicode_minus']=False


_translate = QtCore.QCoreApplication.translate


from geopytool.ImportDependence import *
from geopytool.CustomClass import *


class MyTwoD_Grey(AppForm):
    Lines = []
    Tags = []
    FileName_Hint=''
    description = 'TwoD'
    unuseful = ['Name',
                'Mineral',
                'Author',
                'DataType',
                'Label',
                'Marker',
                'Color',
                'Size',
                'Alpha',
                'Style',
                'Width',
                'Tag']



    def __init__(self, parent=None,  DataFiles =[pd.DataFrame()],DataLocation='DataLocation'):
        QMainWindow.__init__(self, parent)
        self.FileName_Hint=''
        self.DataFiles = DataFiles
        self.DataLocation = DataLocation
        self.Labels = self.getFileName(self.DataLocation)
        self.setWindowTitle('TwoD Data Visualization')


        self.items = []
        if (len(self.DataFiles) > 0):
            self._changed = True
            # print('DataFrame recieved to TwoD')

        self.create_main_frame()

    def create_main_frame(self):
        self.resize(800,600)
        self.main_frame = QWidget()
        self.dpi = 128
        self.setWindowTitle('TwoD Data Visualization')

        self.fig = plt.figure(figsize=(12, 8))
        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.1, right=0.6, top=0.9)

        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        #self.axes = Axes3D(self.fig, elev=-150, azim=110)
        self.axes = self.fig.add_subplot(111)

        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls

        self.log_cb = QCheckBox('&Log')
        self.log_cb.setChecked(True)
        self.log_cb.stateChanged.connect(self.TwoD)  # int

        self.legend_cb = QCheckBox('&Legend')
        self.legend_cb.setChecked(True)
        self.legend_cb.stateChanged.connect(self.TwoD)  # int

        self.y_flip_cb = QCheckBox('&Flip Y')
        self.y_flip_cb.setChecked(False)
        self.y_flip_cb.stateChanged.connect(self.TwoD)  # int

        self.x_flip_cb = QCheckBox('&Flip X')
        self.x_flip_cb.setChecked(False)
        self.x_flip_cb.stateChanged.connect(self.TwoD)  # int


        self.cb_list=[]

        self.setter_list=[]


        for i in range(len(self.Labels)):

            tmp_cb = QCheckBox(self.Labels[i])
            tmp_cb.setChecked(True)
            self.cb_list.append(tmp_cb)

            tmp_setter = QLineEdit(self)
            tmp_setter.setText('Alpha 0-1')
            self.setter_list.append(tmp_setter)


        w = self.width()
        for i in self.cb_list:
            i.setFixedWidth(w / 8)
            i.stateChanged.connect(self.TwoD)  # int

        for i in self.setter_list:
            i.setFixedWidth(w / 8)
            i.textChanged[str].connect(self.TwoD)  # int

        self.save_img_button = QPushButton('&Save Image')
        self.save_img_button.clicked.connect(self.saveImgFile)
        #self.save_img_button.clicked.connect(self.exportScene)

        self.vbox = QVBoxLayout()
        self.hbox1 =QHBoxLayout()
        self.hbox2 =QHBoxLayout()
        self.hbox3 =QHBoxLayout()

        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        self.hbox1.addWidget(self.legend_cb)
        self.hbox1.addWidget(self.log_cb)
        self.hbox1.addWidget(self.x_flip_cb)
        self.hbox1.addWidget(self.y_flip_cb)
        self.hbox1.addWidget(self.save_img_button)

        for i in self.cb_list:
            self.hbox2.addWidget(i)

        for i in self.setter_list:
            self.hbox3.addWidget(i)

        self.vbox.addLayout(self.hbox1)
        self.vbox.addLayout(self.hbox2)
        self.vbox.addLayout(self.hbox3)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)



    def TwoD(self):

        self.FileName_Hint=''
        self.axes.clear()
        self.WholeData = []

        all_matshow =[]

        for k in self.cb_list:
            if k.isChecked():

                text= k.text()



                self.FileName_Hint+=text
                #print(text)

                for i in range(len(self.DataFiles)):
                    ItemsAvalibale = self.DataFiles[i].columns.values.tolist()
                    if 'Label' in ItemsAvalibale:
                        self.DataFiles[i] = self.DataFiles[i].set_index('Label')
                    dataframe = self.DataFiles[i]
                    dataframe = dataframe.dropna(axis=1, how='all')
                    ItemsToTest = ['Number', 'Tag', 'Name', 'Author', 'DataType', 'Marker', 'Color', 'Size', 'Alpha',
                                   'Style', 'Width']
                    for j in ItemsToTest:
                        if j in ItemsAvalibale:
                            dataframe = dataframe.drop(j, 1)

                    dataframe = dataframe.apply(pd.to_numeric, errors='coerce')
                    dataframe = dataframe.dropna(axis='columns')
                    dataframe = pd.DataFrame(dataframe.values)

                    if (self.log_cb.isChecked()):
                        zs = np.log10(dataframe.values)
                    else:
                        zs = dataframe.values




                    # print(type(zs),zs.shape)
                    zs = np.matrix(zs)
                    #print(type(zs), zs.shape)

                    #print( self.Labels[i])

                    alpha = 1

                    try:
                        tmp_alpha = float(self.setter_list[i].text())
                        if 0 <= tmp_alpha <= 1:
                            alpha = tmp_alpha
                    except:
                        pass

                    if (text==self.Labels[i]):
                        #self.axes.matshow(zs, aspect='auto', origin='lower', cmap=self.PureColors[i], alpha=alpha)

                        #m = mean(zs)
                        #new_zs = zs/m
                        #print(new_zs)

                        self.axes.imshow(zs, interpolation='nearest',aspect='auto', origin='lower', cmap= 'Greys', alpha=alpha)


                        #tmp_matshow= plt.matshow(zs, aspect='auto', origin='lower', cmap=self.PureColors[i], alpha=alpha)
                        #all_matshow.append(tmp_matshow)
                        #print(self.PureColor[i])
                        self.axes.scatter(-1,-1,  marker= 'o', s=20,c='grey',alpha= 1, label=self.Labels[i],edgecolors='black')
                        self.axes.set_xlim(left=0)
                        self.axes.set_ylim(bottom=0)




        if (self.legend_cb.isChecked()):
            self.axes.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0, prop=fontprop)


        #x_max,y_max= 0,0
        xlim=self.axes.get_xlim()
        ylim=self.axes.get_ylim()


        print(xlim,ylim )

        if (self.x_flip_cb.isChecked()):
            self.axes.set_xlim(max(xlim),min(xlim))
        else:
            self.axes.set_xlim(min(xlim), max(xlim))

        if (self.y_flip_cb.isChecked()):
            self.axes.set_ylim(max(ylim),min(ylim))
        else:
            self.axes.set_ylim(min(ylim),max(ylim))

        self.canvas.draw()
        self.show()

    def saveImgFile(self):
        ImgFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                         '文件保存',
                                                         'C:/'+self.FileName_Hint,
                                                         'pdf Files (*.pdf);;SVG Files (*.svg);;PNG Files (*.png)')  # 设置文件扩展名过滤,注意用双分号间隔

        if (ImgFileOutput != ''):
            self.canvas.print_figure(ImgFileOutput, dpi=300)
            #self.canvas.print_raw(ImgFileOutput, dpi=300)







from geopytool.ImportDependence import *
from geopytool.CustomClass import *


class Cluster(AppForm):
    Lines = []
    Tags = []
    description = 'Cluster diagram'
    unuseful = ['Name',
                'Mineral',
                'Author',
                'DataType',
                'Label',
                'Marker',
                'Color',
                'Size',
                'Alpha',
                'Style',
                'Width',
                'Tag']

    corr1 = pd.DataFrame()
    corr2 = pd.DataFrame()

    def __init__(self, parent=None, df=pd.DataFrame()):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle('Cluster Data')

        self.items = []
        self._df = df
        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved to Cluster')

        self.raw = df
        self.rawitems = self.raw.columns.values.tolist()

        for i in self.rawitems:
            if i not in self.unuseful:
                self.items.append(i)
            else:
                pass

        self.create_main_frame()
        self.create_status_bar()


    def showResultUp(self):

        self.tablepopUp = TableViewer(df=self.corr1, title='Correlation Matrix of Columns (Top Matrix)')
        self.tablepopUp.show()

    def showResultLeft(self):
        self.tablepopLeft = TableViewer(df=self.corr2, title='Correlation Matrix of Rows (Left Matrix)')
        self.tablepopLeft.show()

    def create_main_frame(self):
        self.resize(800,800)
        self.main_frame = QWidget()
        self.dpi = 128
        self.setWindowTitle('Cluster Figure')

        self.fig = plt.figure(figsize=(12, 12))
        self.fig.subplots_adjust(hspace=0.5, wspace=0.5, left=0.1, bottom=0.1, right=0.9, top=0.9)

        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        #self.axes = self.fig.add_subplot(111)
        # self.axes.hold(False)

        # Create the navigation toolbar, tied to the canvas

        #self.tableView = CustomQTableView(self.main_frame)
        #self.tableView.setObjectName('tableView')
        #self.tableView.setSortingEnabled(True)

        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls
        self.save_button = QPushButton('&Save')

        # self.save_button.clicked.connect(self.saveDataFile)

        self.save_button.clicked.connect(self.saveImgFile)

        self.stat_button = QPushButton('&Stat')
        self.stat_button.clicked.connect(self.Stat)

        self.show_data_buttonUp = QPushButton('&Show Correlation Matrix of Columns (Top Matrix)')
        self.show_data_buttonUp.clicked.connect(self.showResultUp)
        self.show_data_buttonLeft = QPushButton('&Show Correlation Matrix of Rows (Left Matrix)')
        self.show_data_buttonLeft.clicked.connect(self.showResultLeft)


        self.ShowValue_cb = QCheckBox('&Value Off')
        self.ShowValue_cb.setChecked(False)
        self.ShowValue_cb.stateChanged.connect(self.Cluster)  # int

        self.ShowCorr_cb = QCheckBox('&Show Correlation Matrix on Plot')
        self.ShowCorr_cb.setChecked(False)
        self.ShowCorr_cb.stateChanged.connect(self.Cluster)  # int


        #
        # Layout with box sizers
        #
        self.hbox = QHBoxLayout()

        for w in [self.save_button,self.stat_button, self.ShowCorr_cb,self.show_data_buttonLeft,self.show_data_buttonUp]:
            self.hbox.addWidget(w)
            self.hbox.setAlignment(w, Qt.AlignVCenter)

        self.vbox = QVBoxLayout()

        self.vbox.addWidget(self.mpl_toolbar)
        self.vbox.addWidget(self.canvas)
        #self.vbox.addWidget(self.tableView)

        self.vbox.addLayout(self.hbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

    def Read(self, inpoints):
        points = []
        for i in inpoints:
            points.append(i.split())

        result = []
        for i in points:
            for l in range(len(i)):
                a = float((i[l].split(','))[0])
                a = a * self.x_scale

                b = float((i[l].split(','))[1])
                b = (self.height_load - b) * self.y_scale

                result.append((a, b))
        return (result)

    def Cluster(self):

        self.fig.clear()
        self.WholeData = []
        ItemsAvalibale = self._df.columns.values.tolist()


        if 'Label' in ItemsAvalibale:
            self._df = self._df.set_index('Label')

        dataframe = self._df

        dataframe =  dataframe.dropna(axis=1,how='all')



        ItemsToTest = ['Number', 'Tag', 'Name', 'Author', 'DataType', 'Marker', 'Color', 'Size', 'Alpha',
                       'Style', 'Width']

        for i in ItemsToTest:
            if i in ItemsAvalibale:
                dataframe = dataframe.drop(i, 1)



        dataframe2 = dataframe.T

        if self.ShowCorr_cb.isChecked():

            axup = self.fig.add_axes([0.53, 0.88, 0.38, 0.1])

        else:

            axup = self.fig.add_axes([0.3, 0.75, 0.5, 0.1])


        corr1 = 1 - dataframe.corr()


        able =True

        try:
            corr_condensed1 = hc.distance.squareform(corr1)  # convert to condensed
        except:
            able =False
            pass


        if able == False:
            reply = QMessageBox.information(self,'Warning','Your Data Failed to Cluster.\n Please remove Non Valued items and Blanks!')


        else:
            z1 = hc.linkage(corr_condensed1, method='average')
            Z1 = hc.dendrogram(abs(z1),labels=corr1.columns)

            axup.set_xticks([])
            axup.set_yticks([])

            if self.ShowCorr_cb.isChecked():
                axmatrixup = self.fig.add_axes([0.53, 0.49, 0.38, 0.38])

                imup = axmatrixup.matshow(corr1, aspect='auto', origin='lower', cmap='GnBu')

                axmatrixup.set_xticks([])
                axmatrixup.set_xticklabels([], minor=False,prop=fontprop)
                axmatrixup.xaxis.set_label_position('bottom')
                axmatrixup.xaxis.tick_bottom()

                axmatrixup.set_yticks([])
                axmatrixup.set_yticklabels([], minor=False,prop=fontprop)
                axmatrixup.yaxis.set_label_position('right')
                axmatrixup.yaxis.tick_right()

                axleft = self.fig.add_axes([0.0,0.1,0.13,0.38])

            else:

                axleft = self.fig.add_axes([0.09, 0.1, 0.15, 0.6])


            # Compute and plot second dendrogram.

            corr2 = 1- dataframe2.corr()
            corr_condensed2 = hc.distance.squareform(corr2)  # convert to condensed
            z2 = hc.linkage(corr_condensed2, method='average')
            Z2 = hc.dendrogram(abs(z2),labels=corr2.columns,orientation='left')

            axleft.set_xticks([])
            axleft.set_yticks([])

            if self.ShowCorr_cb.isChecked():
                axmatrixleft = self.fig.add_axes([0.14, 0.1, 0.38, 0.38])
                imleft = axmatrixleft.matshow(corr2, aspect='auto', origin='lower', cmap='GnBu')


                axmatrixleft.set_xticks([])
                axmatrixleft.set_xticklabels([], minor=False)
                axmatrixleft.xaxis.set_label_position('bottom')
                axmatrixleft.xaxis.tick_bottom()

                axmatrixleft.set_yticks([])
                axmatrixleft.set_yticklabels([], minor=False)
                axmatrixleft.yaxis.set_label_position('right')
                axmatrixleft.yaxis.tick_right()

                #Plot distance matrix.
                axmatrix = self.fig.add_axes([0.53, 0.1, 0.38, 0.38])

            else:
                axmatrix = self.fig.add_axes([0.3, 0.1, 0.5, 0.6])

            idx1 = Z1['leaves']
            idx2 = Z2['leaves']

            done = np.ones(dataframe.shape)
            a = np.dot(done, corr1)

            b = np.dot(a.T, corr2)

            D = b.T

            D = D[idx2, :]
            D = D[:, idx1]
            im = axmatrix.matshow(D, aspect='auto', origin='lower', cmap='Blues')


            XLabelList = [corr1.columns[i] for i in idx1]
            YLabelList = [corr2.columns[i] for i in idx2]

            XstickList=[i for i in range(len(XLabelList))]
            YstickList=[i for i in range(len(YLabelList))]


            #myfont = matplotlib.font_manager.FontProperties(fname=r'\wqy.ttf', size=14)
            #wqy.ttf
            fontprop = font_manager.FontProperties(family='sans-serif',
                                                   size=9,
                                                   fname=ttfFontProp.fname,
                                                   stretch=ttfFontProp.stretch,
                                                   style=ttfFontProp.style,
                                                   variant=ttfFontProp.variant,
                                                   weight=ttfFontProp.weight)

            axmatrix.set_xticks(XstickList)
            axmatrix.set_xticklabels( XLabelList,fontproperties =fontprop,  minor=False)



            axmatrix.set_yticks(YstickList)
            axmatrix.set_yticklabels( YLabelList,fontproperties =fontprop,  minor=False)

            if self.ShowCorr_cb.isChecked():
                pass
                axmatrix.xaxis.set_label_position('bottom')
                axmatrix.xaxis.tick_bottom()
                axmatrix.yaxis.set_label_position('right')
                axmatrix.yaxis.tick_right()
            else:
                axmatrix.xaxis.set_label_position('top')
                axmatrix.xaxis.tick_top()
                axmatrix.yaxis.set_label_position('left')
                axmatrix.yaxis.tick_left()


            plt.xticks(rotation=-90, fontsize=6)
            plt.yticks(fontsize=6)

            #axcolor1 = self.fig.add_axes([0.3, 0.54, 0.02, 0.4])

            # Plot colorbar.

            #plt.colorbar(im, cax=axcolor1)




            '''
            if self.ShowValue_cb.isChecked():
                self.ShowValue_cb.setText('&Value On')
                for (j, i), label in np.ndenumerate(1-corr1):
                    axmatrixup.text(i, j, "%.2f" % label, ha='center', va='center')
                for (j, i), label in np.ndenumerate(1-corr2):
                    axmatrixleft.text(i, j, "%.2f" % label, ha='center', va='center')
                for (j, i), label in np.ndenumerate(D):
                    axmatrix.text(i, j, "%.2f" % label, ha='center', va='center')

            else:
                self.ShowValue_cb.setText('&Value Off')
                
            if self.ShowCorr_cb.isChecked():
                self.ShowCorr_cb.setText('&Hide Correlation Matrix')
            else:
                self.ShowCorr_cb.setText('&Show Correlation Matrix')
            
            '''


            self.corr1=1-corr1
            self.corr2=1-corr2

            self.canvas.draw()
            self.show()



    def Stat(self):

        df=self._df


        m = ['Width', 'Style', 'Alpha', 'Size', 'Color', 'Marker', 'Author']
        for i in m:
            if i in df.columns.values:
                df = df.drop(i, 1)
        #df.set_index('Label', inplace=True)

        items = df.columns.values
        index = df.index.values

        StatResultDict = {}

        for i in items:
            StatResultDict[i] = self.stateval(df[i])

        StdSortedList = sorted(StatResultDict.keys(), key=lambda x: StatResultDict[x]['std'])

        StdSortedList.reverse()

        '''
        for k in sorted(StatResultDict.keys(), key=lambda x: StatResultDict[x]['std']):
            print("%s=%s" % (k, StatResultDict[k]))
        '''



        StatResultDf = pd.DataFrame.from_dict(StatResultDict, orient='index')
        StatResultDf['Items']=StatResultDf.index.tolist()

        self.tablepop = TableViewer(df=StatResultDf,title='Statistical Result')
        self.tablepop.show()

        self.Intro = StatResultDf
        return(StatResultDf)



from geopytool.ImportDependence import *
from geopytool.CustomClass import *

class ZirconCe(QMainWindow):
    _df = pd.DataFrame()
    _changed = False



    xlabel = r'$(r_i/3+r_{Zr}/6)(r_i-r_{Zr})^2 $'
    ylabel = r'$\log_e D_{Zircon/Base}$'

    reference = 'Ballard, J. R., Palin, M. J., and Campbell, I. H., 2002, Relative oxidation states of magmas inferred from Ce(IV)/Ce(III) in zircon: application to porphyry copper deposits of northern Chile: Contributions to Mineralogy and Petrology, v. 144, no. 3, p. 347-364.'

    Elements3 = ['La', 'Ce', 'Pr', 'Nd', 'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu']
    Elements4 = ['Th', 'U', 'Hf', 'Zr', 'Ce4']


    UsedElements3 = []
    UsedElements4 = []

    Ri3 = [1.16, 1.143, 1.126, 1.109, 1.079, 1.066, 1.053, 1.04, 1.027, 1.015, 1.004, 0.994, 0.985, 0.977]
    Ro3 = [0.84 for i in Ri3]

    Ri4 = [1.05,1.00 ,0.83 ,0.840,0.97 ]
    Ro4 = [0.84 for i in Ri4]

    ZirconZr = 497555

    x3=[]
    x4=[]

    Zircon = []
    ZirconCe=[]
    Ce3test = []
    DCe3test = []
    Ce4test = []
    DCe4test = []
    Ce4_3_Ratio = []

    xCe3 = 0.0479981
    xCe4 = 0.00788412

    def __init__(self, parent=None, df=pd.DataFrame()):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle('Oxygen Fugacity Estimation by Ce(IV)/Ce(III) in Zircon (Ballard et al. 2002)')


        for i in range(len(self.Ri3)):
            self.x3.append((self.Ri3[i] / 3 + self.Ro3[i] / 6) * (self.Ri3[i] - self.Ro3[i]) * (self.Ri3[i] - self.Ro3[i]))

            if self.Elements3[i]=='Ce':
                self.xCe3=((self.Ri3[i] / 3 + self.Ro3[i] / 6) * (self.Ri3[i] - self.Ro3[i]) * (self.Ri3[i] - self.Ro3[i]))

        for i in range(len(self.Ri4)):
            self.x4.append((self.Ri4[i] / 3 + self.Ro4[i] / 6) * (self.Ri4[i] - self.Ro4[i]) * (self.Ri4[i] - self.Ro4[i]))
            if self.Elements3[i] == 'Ce4':
                self.xCe4=((self.Ri4[i] / 3 + self.Ro4[i] / 6) * (self.Ri4[i] - self.Ro4[i]) * (self.Ri4[i] - self.Ro4[i]))

        self._df = pd.DataFrame()
        self.raw = pd.DataFrame()

        self._df = df
        self.raw = df

        if (len(df) > 0):
            self._changed = True
            # print('DataFrame recieved')

        self.create_main_frame()
        self.create_status_bar()

    def save_plot(self):
        file_choices = 'pdf Files (*.pdf);;SVG Files (*.svg);;PNG Files (*.png)'

        path = QFileDialog.getSaveFileName(self,
                                           'Save file', '',
                                           file_choices)
        if path:
            self.canvas.print_figure(path, dpi=self.dpi)
            self.statusBar().showMessage('Saved to %s' % path, 2000)

    def create_main_frame(self):

        self.resize(1200, 800)
        self.main_frame = QWidget()
        self.dpi = 128
        self.fig, self.axes = plt.subplots(1, 2, figsize=(12.0, 12.0), dpi=self.dpi)
        self.fig.subplots_adjust(hspace=0.1, wspace=0.1, left=0.1, bottom=0.2, right=0.9, top=0.9)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        # Other GUI controls
        self.save_img_button = QPushButton('&Save Figure')
        self.save_img_button.clicked.connect(self.saveImgFile)

        self.show_data_button = QPushButton('&Show Result')
        self.show_data_button.clicked.connect(self.showResult)

        self.save_data_button = QPushButton('&Save Result')
        self.save_data_button.clicked.connect(self.saveResult)

        #
        # Layout with box sizers
        #
        self.hbox = QHBoxLayout()

        for w in [self.save_img_button, self.show_data_button, self.save_data_button]:
            self.hbox.addWidget(w)
            self.hbox.setAlignment(w, Qt.AlignVCenter)

        self.vbox = QVBoxLayout()
        self.vbox.addWidget(self.mpl_toolbar)

        self.vbox.addWidget(self.canvas)

        self.vbox.addLayout(self.hbox)

        self.textbox = GrowingTextEdit(self)
        self.textbox.setText(self.reference)
        self.vbox.addWidget(self.textbox)

        self.main_frame.setLayout(self.vbox)
        self.setCentralWidget(self.main_frame)

    def create_status_bar(self):
        self.textbox = QLineEdit(self)
        self.textbox.setText('Reference：' + '\n' + self.reference)
        self.statusBar().addWidget(self.textbox, 1)

    def add_actions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def saveImgFile(self):
        ImgFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                         '文件保存',
                                                         'C:/',
                                                         'pdf Files (*.pdf);;SVG Files (*.svg);;PNG Files (*.png)')  # 设置文件扩展名过滤,注意用双分号间隔

        if (ImgFileOutput != ''):
            self.canvas.print_figure(ImgFileOutput, dpi=300)

    def saveResult(self):
        DataFileOutput, ok2 = QFileDialog.getSaveFileName(self,
                                                          '文件保存',
                                                          'C:/',
                                                          'Excel Files (*.xlsx);;CSV Files (*.csv)')  # 数据文件保存输出

        if (DataFileOutput != ''):

            if ('csv' in DataFileOutput):
                self.newdf.to_csv(DataFileOutput, sep=',', encoding='utf-8')

            elif ('xls' in DataFileOutput):
                self.newdf.to_excel(DataFileOutput, encoding='utf-8')

    def showResult(self):

        self.tablepop = TableViewer(df=self.newdf, title='Zircon Ce4_3 Ratio Result')
        self.tablepop.show()

    def create_action(self, text, slot=None, shortcut=None,
                      icon=None, tip=None, checkable=False,
                      signal='triggered()'):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(':/%s.png' % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            action.triggered.connect(slot)
        if checkable:
            action.setCheckable(True)
        return action


    def MultiBallard(self):

        self.axes[0].clear()
        self.axes[1].clear()

        self.axes[0].spines['right'].set_color('none')
        self.axes[0].spines['top'].set_color('none')

        self.axes[1].spines['right'].set_color('none')
        self.axes[1].spines['top'].set_color('none')

        self.axes[0].set_xlabel(self.xlabel)
        self.axes[0].set_ylabel(self.ylabel)
        self.axes[1].set_xlabel(self.xlabel)
        self.axes[1].set_ylabel(self.ylabel)






        self.items = self.raw.columns.values.tolist()

        self.rows = self.raw.index.values.tolist()

        DataX3=[]
        DataX4=[]

        Ybase3=[]
        Ybase4=[]


        self.Base = 0

        for i in range(len(self.raw)):
            if (self.raw.at[i, 'DataType'] == 'Base'):
                self.Base = i
                self.BaseCe = self.raw.at[i, 'Ce']
                self.BaseZr = self.raw.at[i, 'Zr']

            elif (self.raw.at[i, 'DataType'] == 'Zircon'):
                self.Zircon.append(i)


        for i in self.items:
            if i in self.Elements3:
                self.UsedElements3.append(i)
                DataX3.append(self.x3[self.Elements3.index(i)])
                Ybase3.append(self.raw.at[self.Base,i])

            elif i in self.Elements4:
                self.UsedElements4.append(i)
                DataX4.append(self.x4[self.Elements4.index(i)])
                Ybase4.append(self.raw.at[self.Base,i])




        #np.log(yi / ybase)



        print(self.rows)

        self.FittedData=[]



        print('\n DataX3 ',len(DataX3),'\n DataX4',len(DataX4))


        for i in self.rows:
            tmpy3 = []
            tmpy4 = []
            fittmpy3 = []
            fittmpy4 = []
            fitDataX3 = []
            fitDataX4 = []
            if i != self.Base:
                self.ZirconCe.append(self.raw.at[i, 'Ce'])


                for j in self.UsedElements3:
                    if len(tmpy3)<len(DataX3):
                        tmpy3.append(np.log(self.raw.at[i,j]/Ybase3[self.UsedElements3.index(j)]))
                for j in self.UsedElements4:
                    if len(tmpy4)<len(DataX4):
                        tmpy4.append(np.log(self.raw.at[i,j]/Ybase4[self.UsedElements4.index(j)]))


                print('\n y3 ', len(tmpy3), '\n y4', len(tmpy4))



                for q in self.UsedElements3:
                    if q != 'Ce':
                        fitDataX3.append(DataX3[self.UsedElements3.index(q)])
                        fittmpy3.append(tmpy3[self.UsedElements3.index(q)])

                for q in self.UsedElements4:
                    if q != 'Ce4':
                        fitDataX4.append(DataX4[self.UsedElements4.index(q)])
                        fittmpy4.append(tmpy4[self.UsedElements4.index(q)])


                print(len(fitDataX3), len(fittmpy3))
                print(len(fitDataX4), len(fittmpy4))

                tmpz3 = np.polyfit(fitDataX3, fittmpy3, 1)
                tmpz4 = np.polyfit(fitDataX4, fittmpy4, 1)



                Xline3 = np.linspace(min(fitDataX3), max(fitDataX3), 30)
                p3 = np.poly1d(tmpz3 )
                Yline3 = p3(Xline3)





                Xline4 = np.linspace(min(fitDataX4), max(fitDataX4), 30)
                p4 = np.poly1d(tmpz4 )
                Yline4 = p3(Xline4)




                self.Ce3test.append(np.power(np.e, p3(self.xCe3) + np.log(self.BaseCe)))
                self.DCe3test.append(np.power(np.e, p3(self.xCe3)))
                self.Ce4test.append(np.power(np.e, p4(self.xCe4) + np.log(self.BaseCe)))
                self.DCe4test.append(np.power(np.e, p4(self.xCe4)))



                self.axes[0].scatter(DataX3, tmpy3, s=3,color='blue', alpha=0.3, label='')
                self.axes[1].scatter(DataX4, tmpy4, s=3,color='red', alpha=0.3, label='')

                self.axes[0].plot(Xline3, Yline3, 'b-', alpha=0.3)
                self.axes[1].plot(Xline4, Yline4, 'r-', alpha=0.3)

                if i ==1:
                    for k in range(len(DataX3)):
                        self.axes[0].annotate(self.UsedElements3[k], xy=(DataX3[k], tmpy3[k]), fontsize=6, xytext=(16, 16),
                                              textcoords='offset points',
                                              ha='right', va='bottom',
                                              bbox=dict(boxstyle='round,pad=0.2', fc='blue', alpha=0.3),
                                              arrowprops=dict(arrowstyle='->', connectionstyle='arc3,rad=0'))
                    for k in range(len(DataX4)):
                        self.axes[1].annotate(self.UsedElements4[k], xy=(DataX4[k], tmpy4[k]), fontsize=6, xytext=(16, 16),
                                              textcoords='offset points',
                                              ha='right', va='bottom',
                                              bbox=dict(boxstyle='round,pad=0.2', fc='red', alpha=0.3),
                                              arrowprops=dict(arrowstyle='->', connectionstyle='arc3,rad=0'))

                tmpy3 = []
                tmpy4 = []
                fittmpy3 = []
                fittmpy4 = []
                fitDataX3 = []
                fitDataX4 = []
        self.canvas.draw()


        self.DataToWrite = [
            ['Zircon Sample Label', 'Zircon Ce4_3 Ratio', 'Melt Ce4_3 Ratio', 'DCe4', 'DCe3', 'DCe Zircon/Melt'], ]
        for i in range(len(self.ZirconCe)):
            TMP = self.raw.at[self.Zircon[i], 'Label']
            ZirconTmp = (self.BaseCe - self.ZirconCe[i] / self.DCe3test[i]) / (
                    self.ZirconCe[i] / self.DCe4test[i] - self.BaseCe)
            MeltTmp = (self.ZirconCe[i] - self.Ce3test[i]) / self.Ce3test[i] * self.DCe3test[i] / self.DCe4test[
                i]
            self.Ce4_3_Ratio.append(ZirconTmp)

            if len(self.DataToWrite) < len(DataX3):
                self.DataToWrite.append([TMP, ZirconTmp, MeltTmp, self.DCe4test[i], self.DCe3test[i], self.ZirconCe[i] / self.BaseCe])
        self.newdf = pd.DataFrame(self.DataToWrite)





